/*
 
 iChatUnleash.bundle - Overriding iChat's G3 Restrictions
 http://www.liquidx.net/ichatunleash/
 
 Copyright (c) 2006, Alastair Tse <alastair@liquidx.net>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 
 Redistributions in binary form must reproduce the above copyright notice, this
 list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.
 
 Neither the Alastair Tse nor the names of its contributors may
 be used to endorse or promote products derived from this software without 
 specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
 */

#import "VCRules_Unleashed.h"

@implementation VCRules (Unleashed)

+ (BOOL) canSupportH264DecodeUnleashed
{
	BOOL can = [VCRules canSupportH264DecodeUnleashed];
	NSLog(@"iChatUnleashed: calling overrided canSupportH264Decode: %d", can);	
	return YES;
}

+ (BOOL) canSupportH264EncodeUnleashed
{
	BOOL can = [VCRules canSupportH264EncodeUnleashed];
	NSLog(@"iChatUnleashed: calling overrided canSupportH264Encode: %d", can);
	return YES;
}

+ (BOOL) isHardwareSuitableForFiltersUnleashed
{
	BOOL isit = [VCRules isHardwareSuitableForFiltersUnleashed];
	NSLog(@"iChatUnleashed: calling isHardwareSuitableForFiltersUnleashed -> %d", isit);
	return isit;
}

+ (NSString *)unleashedRulesForVideoCodec:(int)fp8 withCurrentUserCount:(int)fp12 withBitrateLimit:(int)fp16
{
	NSString * rules = [VCRules unleashedRulesForVideoCodec:fp8 withCurrentUserCount:fp12 withBitrateLimit:fp16];
	NSLog(@"iChatUnleashed: calling rulesForVideoCodec: %@", rules);
	return rules;
}

+ (NSString *)unleashedRulesForH263WithCurrentUserCount:(int)fp8 withBitrateLimit:(int)fp12
{
	id rules = [VCRules unleashedRulesForH263WithCurrentUserCount:fp8 withBitrateLimit:fp12];
	NSLog(@"iChatUnleashed: calling rulesForH263WithCurrentUserCount: %@", rules);
	return rules;
}

+ (NSString *)unleashedRulesForH264WithCurrentUserCount:(int)fp8 withBitrateLimit:(int)fp12
{
	// G4: 15:160:120:160:120
	// G3: 10:176:144:176:144
	
	NSString * rules = [VCRules unleashedRulesForH264WithCurrentUserCount:fp8 withBitrateLimit:fp12];
	NSLog(@"iChatUnleashed: calling rulesForH264WithCurrentUserCount: %@", rules) ;
	return rules;
}

+ (unsigned int)unleashedMinAudioBandwidth
{
	unsigned int bandwidth = [VCRules unleashedMinAudioBandwidth];
	NSLog(@"iChatUnleashed: minAudioBandwidth: %d", bandwidth);
	return bandwidth;
}

+ (unsigned int)unleashedMinVideoBandwidth
{
	unsigned int bandwidth = [VCRules unleashedMinVideoBandwidth];
	NSLog(@"iChatUnleashed: minVideoBandwidth: %d", bandwidth);
	return bandwidth;
}

+ (unsigned short)unleashedCutoff15FPS
{
	unsigned short cutoff = [VCRules unleashedCutoff15FPS];
	NSLog(@"iChatUnleashed: cutOff15FPS: %d", cutoff);
	return cutoff;
}
+ (unsigned int)unleashedMagicRTCBandwidth
{
	unsigned int bandwidth = [VCRules unleashedMagicRTCBandwidth];
	NSLog(@"iChatUnleashed: magicRTCBandwidth: %d", bandwidth);
	return bandwidth;
}

+ (unsigned int)unleashedMinCPUSpeedForH264
{
	unsigned int speed = [VCRules unleashedMinCPUSpeedForH264];
	NSLog(@"iChatUnleashed: minCPUSpeedForH264: %d", speed);
	return speed;
}

+ (void)unleashedBestParametersForFramerate:(int *)fp8 imageSize:(int *)fp12 forTransportType:(int)fp16
{
	int frameRate;
	int imageSize;
	
	// G4: framerate: 15 imagesize: 1
	// G3: framerate: 10 iamgesize: 1
	
	[VCRules unleashedBestParametersForFramerate:&frameRate imageSize:&imageSize forTransportType:fp16];
	NSLog(@"iChatUnleashed: bestParametersForFramerate:imageSize:forTransportType: %d %d",
		  frameRate, imageSize);
	
	memcpy(fp8, &frameRate, sizeof(int));
	memcpy(fp12, &imageSize, sizeof(int));
	return;
}

+ (NSString *) unleashedGenerateHardwareInfoForSDP
{
	// G4 : hwi:70:1:999
	// G3 : hwi:133:1:499
	// explain: hwi:(mem?):true:MHz
	
	NSString * hardwareInfo = [VCRules unleashedGenerateHardwareInfoForSDP];
	NSLog(@"iChatUnleashed: generateHardwareInfoForSDP: %@", hardwareInfo);
	NSMutableArray *components = [NSMutableArray arrayWithArray:[hardwareInfo componentsSeparatedByString:@":"]];
	int mhz = [[components objectAtIndex:3] intValue];
	if (mhz > 850)
		[components replaceObjectAtIndex:3 withObject:@"500"];
	[components replaceObjectAtIndex:1 withObject:@"133"];
	hardwareInfo = [components componentsJoinedByString:@":"];
	NSLog(@"iChatUnleashed: generateHardwareInfoForSDP: (Modified) %@", hardwareInfo);
	return hardwareInfo;
}


/*
 [VCRules bestParametersForFramerate:imageSize:forTransportType:]
+ (struct _NSSize)applyImageSizeDowngradeRuleForPayload:(int)fp8 currentSize:(struct _NSSize)fp12 bitrate:(int)fp20 pFramerate:(int *)fp24 inMultiway:(BOOL)fp28;
+ (id)generateHardwareInfoForSDP;
+ (void)parseSDPHardwareInfo:(id)fp8 forMachineType:(int *)fp12 forCPUCount:(int *)fp16 forCPUSpeed:(int *)fp20;
+ (void)getMultipointInfoForPayload:(int)fp8 imageWidth:(int *)fp12 imageHeight:(int *)fp16 framerate:(int *)fp20;
*/

/* Notes:

 need to find out where that is used: ProcessorSpeedMHz

#0  0x9814153c in ProcessorSpeedMHz ()
#1  0x98143094 in DeviceManager_Create ()
#2  0x98142f04 in Conference_Create ()
#3  0x9813f9f4 in -[VideoConferenceController initWithRectTexture:] ()
#4  0x9813f71c in -[VideoConferenceMultiController initWithRectTexture:] ()

#0  0x9814153c in ProcessorSpeedMHz ()#
1  0x98141490 in getHardwareInfo ()
#2  0x98148208 in +[VCRules generateHardwareInfoForSDP] ()
#3  0x98147d14 in -[VideoConferenceMultiController(helper) determineFocusCapability:] ()
#4  0x98147ab0 in -[VideoConferenceMultiController vcCapabilities] ()

*/

@end
