/*
 
 iChatUnleash.bundle - Overriding iChat's G3 Restrictions
 http://www.liquidx.net/ichatunleash/
 
 Copyright (c) 2006, Alastair Tse <alastair@liquidx.net>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 
 Redistributions in binary form must reproduce the above copyright notice, this
 list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.
 
 Neither the Alastair Tse nor the names of its contributors may
 be used to endorse or promote products derived from this software without 
 specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
 */

#import <CoreServices/CoreServices.h>
#import "iChatUnleash.h"
#import <objc/objc-class.h>
#import "mach_override.h"

BOOL ClassMethodSwizzle(Class aClass, SEL orig_sel, SEL alt_sel)
{
    Method orig_method = nil, alt_method = nil;
	
    // First, look for the methods
    orig_method = class_getClassMethod(aClass, orig_sel);
    alt_method = class_getClassMethod(aClass, alt_sel);
	
    // If both are found, swizzle them
    if ((orig_method != nil) && (alt_method != nil))
	{
        char *temp1;
        IMP temp2;
		
        temp1 = orig_method->method_types;
        orig_method->method_types = alt_method->method_types;
        alt_method->method_types = temp1;
		
        temp2 = orig_method->method_imp;
        orig_method->method_imp = alt_method->method_imp;
        alt_method->method_imp = temp2;
		return YES;
	}
	return NO;
}

// We define some hidden functions from VideoConference that we really should
// have access to:

BOOL HasAltivec();
int ProcessorSpeedMHz();

int unleashedProcessorSpeedMHz()
{
	OSErr err;
	long speed;
	err = Gestalt (gestaltProcClkSpeed, &speed  );
	if (err)
		return 0;
	
	if (speed < 600)
		return 601;
	return speed;
}


@class FCRules;

@interface iChatUnleash (Private)
+ (iChatUnleash *) sharedInstance;
@end


@implementation iChatUnleash

+ (void) install
{
	iChatUnleash* plugin = [iChatUnleash sharedInstance];
	//BOOL success = NO;
	
	mach_error_t err = mach_override_ptr((void *)&ProcessorSpeedMHz,
										 (void *)&unleashedProcessorSpeedMHz,
										 (void **)NULL);
	if (err)
		NSLog(@"iChatUnleash: Error with mach_override");
		
	NSLog(@"iChatUnleash: Installed");
	/*
	
	success = ClassMethodSwizzle([VCRules class], 
								 @selector(canSupportH264Encode),
								 @selector(canSupportH264EncodeUnleashed));
	if (success)
		NSLog(@"iChatUnleash: canSupportH264Encode: Unleashed");	
	
	success = ClassMethodSwizzle([VCRules class],
								 @selector(canSupportH264Decode),
								 @selector(canSupportH264DecodeUnleashed));	
	*/
}

+ (iChatUnleash *) sharedInstance
{
	static iChatUnleash* plugin = nil;
	if (plugin == nil)
		plugin = [[iChatUnleash alloc] init];
	return plugin;
}

@end
